import Flutter
import UIKit
import CryptoSwift

public class SwiftPrsUtilityPlugin: NSObject, FlutterPlugin {
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "one.press.utility/prs_utility_plugin", binaryMessenger: registrar.messenger())
        let instance = SwiftPrsUtilityPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch (call.method) {
        case CallMethod.pbkdf2.rawValue:
            onPbkdf2(call,result:result)
        default:result(FlutterMethodNotImplemented)
        }
    }
    
    private func onPbkdf2(_ call: FlutterMethodCall, result: (Any?) -> Void) {
        guard let arguments = isArgumentIllegal(call, result: result) else {
            return
        }
        
        let hashVariant = HMAC.Variant.sha256;
        
        let password = arguments["password"] as! String
        let salt = arguments["salt"] as! String
        let iterations = arguments["iterations"] as! Int
        let keyLength = arguments["keyLength"] as! Int
        
        guard let passData = password.data(using: .utf8) else {return}
        
        let saltData = Data(Array<UInt8>(hex: salt))
        
        guard let derivedArray = try? PKCS5.PBKDF2(password: passData.bytes, salt: saltData.bytes
            , iterations: iterations, keyLength: keyLength, variant: hashVariant).calculate() else {return}
        result(derivedArray.toHexString());
        
    }
    
    // only accept map param
    private func isArgumentIllegal(_ call: FlutterMethodCall, result: FlutterResult) -> Dictionary<String, Any?>? {
        guard let arguments = call.arguments as? Dictionary<String, Any?> else {
            result(FlutterError(
                code: ErrorCode.argsError.rawValue,
                message: String.init(format: "arguments in %s method type error.need map", arguments: [call.method]),
                details: nil
            ))
            return nil
        }
        return arguments
    }
}


